<?php
    lt_include( PLOG_CLASS_PATH."class/plugin/pluginbase.class.php" );
    lt_include( PLOG_CLASS_PATH."plugins/subscribe/class/dao/subscriptions.class.php" );
    lt_include( PLOG_CLASS_PATH."plugins/subscribe/class/security/subscribefilter.class.php" );	

    /**
     * Sends notifications to visitors if they sign up for them, such as
     * new comments to a post, new post to a particular category, etc.
     */
    class PluginSubscribe extends PluginBase
    {
        var $_pluginEnabled;
        
        function PluginSubscribe($source = ""){
            $this->PluginBase($source);
            
            $this->id = "subscribe";
            $this->desc = "Sends email notifications to visitors if they ".
                "sign up for them.";
            $this->author = "Jon Daley";
            $this->locales = Array("en_UK");
            $this->version = "20120912";
			if( $source == "admin" )
				$this->initAdmin();
			else
				$this->init();
        }

        function initAdmin(){
                // register our actions
            $this->registerAdminAction("subscribe",
                                       "PluginSubscribeShowConfigAction");
            $this->registerAdminAction("subscribeUpdate",
                                       "PluginSubscribeUpdateConfigAction");
                // register our menu options
			$menu =& Menu::getMenu();
			if(!$menu->entryExists("/menu/controlCenter/manageRecentPlugins"))
				$this->addMenuEntry("/menu/controlCenter",
                                    "manageRecentPlugins", "", "",
                                    true, false);
            $this->addMenuEntry("/menu/controlCenter/manageRecentPlugins",
                                "subscribe_subscriptions",
                                "?op=subscribe", "");
            $this->registerNotification(EVENT_POST_POST_ADD);
        }
        
        function init(){
			$this->registerBlogAction("subscribe",
                                      "PluginSubscribeShowAction");
			$this->registerBlogAction("subscribeRegister",
                                      "PluginSubscribeRegisterAction");
			$this->registerBlogAction("subscribeConfirm",
                                      "PluginSubscribeConfirmAction");
			$this->registerBlogAction("subscribeRemove",
                                      "PluginSubscribeRemoveAction");
                // we care about when posts and comments are posted
            $this->registerNotification(EVENT_POST_POST_ADD);
            $this->registerNotification(EVENT_POST_COMMENT_ADD);
                // We need a filter because the email address is required
                // if the subscribe checkbox is checked.
            $this->registerFilter("SubscribeFilter");
        }

	    function isEnabled(){
	        return $this->_pluginEnabled;
	    }

        function register(){
            $this->_pluginEnabled = $this->blogSettings->getValue("plugin_subscribe_enabled");
            return true;
        }
        
            // create the sql table
        function install(){
            lt_include( PLOG_CLASS_PATH."class/database/db.class.php" );
            
            $fields = "objid INT(10) UNSIGNED NOTNULL,
                  active TINYINT(1) DEFAULT 0,
                  type TINYINT(1) NOTNULL,
                  address VARCHAR(255) NOTNULL,
                  confirm VARCHAR(32)";

            $db =& Db::getDb();
            $tableName = Db::getPrefix()."subscribe";
            
                // create the data dictionary and create
                // the table if necessary
            $dict = NewPDbDataDictionary($db);
            $sqlArray = $dict->ChangeTableSQL($tableName, $fields);
            $result = $dict->ExecuteSQLArray($sqlArray);
            if(!$result)
                die("There was an error creating the tables ".
                    "for the subscribe plugin!");

            $sqlArray = $dict->CreateIndexSQL("id_type_address",
                                              $tableName,
                                              array("objid","type","address"),
                                              array("REPLACE", "UNIQUE"));
                // create the primary key
            $result = $dict->ExecuteSQLArray($sqlArray);
            if(!$result)
                die("There was an error creating the index ".
                    "for the subscribe plugin!");

            return true;
        }

        
        /**
         * This method processes our event notifications
         */        
        function process($eventType, $params){        
            lt_include(PLOG_CLASS_PATH."class/data/textfilter.class.php");
            lt_include(PLOG_CLASS_PATH."class/dao/article.class.php");
            lt_include(PLOG_CLASS_PATH."class/net/requestgenerator.class.php" );	

            $locale = $this->blogInfo->getLocale();
            
            switch($eventType){
              case EVENT_POST_POST_ADD:
                $article = $params["article"];
                if(!$article || $article->getStatus() != POST_STATUS_PUBLISHED)
                    return;

                    // don't send notifications for future posts
                $now = new Timestamp();
                if($article->getDateObject() > $now)
                    return;
                
                $rg = RequestGenerator::getRequestGenerator($this->blogInfo);
                $arr = Subscriptions::getSubscriptions($article->getCategoryIds(),
                                        SUBSCRIBE_POSTS_IN_CATEGORY);
                foreach($arr as $email){
                        // TODO: provide a templating
                        // mechanism for the body of the email
                    Subscriptions::sendMessage($email,
                                               $locale->pr("subscribe_article_published",
                                                           $article->getTopic()),
                                               Textfilter::htmlDecode(
                                                   Textfilter::filterAllHTML(
                                                       $article->getIntroText())) .
                                               "\n\n".$rg->postPermalink($article).
                                               "\n\n".
                                               $locale->pr("subscribe_email_removal",
                                                           Subscriptions::generateConfirmUrl(
                                                               $this->blogInfo,
                                                               "subscribeRemove",
                                                               $email)));
                }
                break;

              case EVENT_POST_COMMENT_ADD:
                lt_include(PLOG_CLASS_PATH.
                             "class/dao/usercomment.class.php");
                lt_include(PLOG_CLASS_PATH.
                             "class/net/requestgenerator.class.php");
                $comment = $params["comment"];
                if($comment->getStatus() != COMMENT_STATUS_NONSPAM)
                    return;
                $article = $comment->getArticle();

                $rg = RequestGenerator::getRequestGenerator($this->blogInfo);
                $arr = Subscriptions::getSubscriptions(
                    array($comment->getArticleId()),
                    SUBSCRIBE_COMMENTS_IN_POST);

                foreach($arr as $email){
                        // TODO: provide a templating
                        // mechanism for the body of the email
                    Subscriptions::sendMessage($email,
                                               $locale->pr("subscribe_comment_on",
                                                           $article->getTopic()),
                                               $locale->pr("subscribe_posted_by",
                                                           $comment->getUserName())."\n".
                                               Textfilter::htmlDecode(
                                                   Textfilter::filterAllHTML(
                                                       $comment->getText())) .
                                               "\n\n".$rg->postPermalink($article).
                                               "\n\n".
                                               $locale->pr("subscribe_email_removal",
                                                           Subscriptions::generateConfirmUrl(
                                                               $this->blogInfo,
                                                               "subscribeRemove",
                                                               $email)));
                }

                $request = HttpVars::getRequest();

                    // Now check to see if this guy wants to receive
                    // future notifications of comments
                if(isset($request["subscribe"])){
                        // We already validated this
                    $emailAddress = $request["userEmail"];

                        // check if they are already logged in (ie. an authenticated commenter)
                    $userInfo = SessionManager::getUserInfoFromSession();  
                    if($emailAddress == "" && $userInfo){
                        $emailAddress = $userInfo->getEmail();
                    }
                    
                        // save subscription info to database
                    if(Subscriptions::addSubscriptions($this->blogInfo,
                                                       array($comment->getArticleId()),
                                                        SUBSCRIBE_COMMENTS_IN_POST,
                                                        $emailAddress))
                    {
                            // send confirmation email
                        if(!Subscriptions::sendConfirmationEmail($this->blogInfo,
                                                                 $emailAddress)){
                                // TODO: can we do anything if there was an error?
                        }
                    }
                    else{
                            // TODO: can we do anything if there was an error?
                    }
                }
                
                break;
              default:
                return;
            }
        }

        function getPluginConfigurationKeys(){
			lt_include( PLOG_CLASS_PATH."class/data/validator/stringvalidator.class.php" );

            return(
                Array(
                    Array( "name" => "plugin_subscribe_enabled", "type" => "boolean"),
                    Array( "name" => "plugin_subscribe_key", "validator" => new StringValidator(), "type" => "string", "allowEmpty" => true ),
                      )
                   );
        }
        
    }
?>